@@ -4,13 +4,12 @@
 
 #include <stdio.h>
 #include <ocidl.h>
+#include <Shlwapi.h>
 
 #include "tCOMUtil.h"
 #include "tLuaCOMException.h"
 #include "tUtil.h"
 
-#define CHK_COM_ERR2(hr) CHK_COM_ERR(hr, tUtil::GetErrorMessage(hr))
-
 
 //////////////////////////////////////////////////////////////////////
 // Construction/Destruction
@@ -146,7 +145,12 @@ ITypeInfo *tCOMUtil::GetDefaultInterfaceTypeInfo(ITypeInfo* pCoClassinfo,
                                                  bool source)
            
 {
-  ITypeInfo* typeinfo = NULL;  //Use this to determine success
+  ITypeInfo* typeinfo = NULL;
+
+  // if the component does not have a dispinterface typeinfo 
+  // for events, we stay with an interface typeinfo
+  ITypeInfo* interface_typeinfo = NULL;
+
   TYPEATTR* pTA = NULL;
   HRESULT hr = S_OK;
 
@@ -187,8 +191,7 @@ ITypeInfo *tCOMUtil::GetDefaultInterfaceTypeInfo(ITypeInfo* pCoClassinfo,
 
           if(FAILED(hr))
           {
-            typeinfo->Release();
-            typeinfo = NULL;
+            COM_RELEASE(typeinfo);
             break;
           }
 
@@ -200,8 +203,16 @@ ITypeInfo *tCOMUtil::GetDefaultInterfaceTypeInfo(ITypeInfo* pCoClassinfo,
           if(typekind == TKIND_DISPATCH)
           {
             // found!
+            COM_RELEASE(interface_typeinfo);
             break;
           }
+          else // hold this pointer. If we do not find
+            // anything better, we stay with this typeinfo
+          {
+            COM_RELEASE(interface_typeinfo);
+            interface_typeinfo = typeinfo;
+            typeinfo = NULL;
+          }
         }
       }
     }
@@ -209,7 +220,10 @@ ITypeInfo *tCOMUtil::GetDefaultInterfaceTypeInfo(ITypeInfo* pCoClassinfo,
     pCoClassinfo->ReleaseTypeAttr(pTA);
   }
 
-  return typeinfo;
+  if(!typeinfo)
+    return interface_typeinfo;
+  else
+    return typeinfo;
 }
 
 
@@ -260,11 +274,10 @@ ITypeInfo *tCOMUtil::GetDispatchTypeInfo(IDispatch* pdisp)
 
   hr = ptypelib->GetTypeInfoOfGuid(iid, &typeinfo_guid);
 
+  COM_RELEASE(ptypelib);
+
   if(FAILED(hr))
-  {
-    ptypelib->Release();
     return typeinfo;
-  }
 
   // verifica se e' dispinterface
   TYPEATTR *ptypeattr_iface = NULL;
@@ -343,7 +356,7 @@ ITypeInfo *tCOMUtil::GetInterfaceTypeInfo(ITypeLib* typelib,
 ITypeLib* tCOMUtil::LoadTypeLibFromProgID(const char* ProgID,
                                           unsigned short major_version)
 {
-  CLSID clsid;
+  CLSID clsid = IID_NULL;
   HRESULT hr = tCOMUtil::ProgID2CLSID(&clsid, ProgID);
 
   if(FAILED(hr))
@@ -382,7 +395,7 @@ ITypeLib *tCOMUtil::LoadTypeLibByName(const char *pcFilename)
 
   hr = LoadTypeLibEx(wcFilename, REGKIND_NONE, &ptlib);
 
-  delete wcFilename;
+  delete[] wcFilename;
   wcFilename = NULL;
 
   if(FAILED(hr))
@@ -465,7 +478,7 @@ ITypeLib* tCOMUtil::LoadTypeLibFromCLSID(CLSID clsid,
 
   // extracts version information
 
-  unsigned short version_major = 0, version_minor = 0;
+  int version_major = 0, version_minor = 0;
 
   int found = 0;
 
@@ -502,8 +515,7 @@ ITypeLib* tCOMUtil::LoadTypeLibFromCLSID(CLSID clsid,
 
   free(bLibID);
 
-
-  CLSID libid;
+  GUID libid = IID_NULL;
   CLSIDFromString(wcTypelib, &libid);
   free(wcTypelib);
 
@@ -522,7 +534,7 @@ ITypeInfo* tCOMUtil::GetCoClassTypeInfo(ITypeLib *typelib, CLSID clsid)
   ITypeInfo* coclassinfo = NULL;
   
   HRESULT hr = typelib->GetTypeInfoOfGuid(clsid, &coclassinfo);
-
+  
   if(FAILED(hr))
     return NULL;
 
@@ -532,11 +544,25 @@ ITypeInfo* tCOMUtil::GetCoClassTypeInfo(ITypeLib *typelib, CLSID clsid)
 HRESULT tCOMUtil::ProgID2CLSID(CLSID *pClsid, const char *ProgID)
 {
   CHECKPARAM(pClsid);
+  
+  // tests whether we already have a CLSID
 
   wchar_t* wcProgId = NULL;
-
   HRESULT hr = S_OK;
 
+  if(ProgID[0] == '{')
+  {
+    wcProgId = (wchar_t*) malloc( (strlen(ProgID) + 1) * sizeof(wchar_t));
+    mbstowcs(wcProgId, ProgID, strlen(ProgID)+1);
+
+    hr = CLSIDFromString(wcProgId, pClsid);
+
+    free(wcProgId);
+    wcProgId = NULL;
+
+    return hr;
+  }
+
   /* converte ProgID para OLESTR */
   wcProgId = (wchar_t*) malloc( (strlen(ProgID) + 1) * sizeof(wchar_t));
   mbstowcs(wcProgId, ProgID, strlen(ProgID)+1);
@@ -549,25 +575,25 @@ HRESULT tCOMUtil::ProgID2CLSID(CLSID *pClsid, const char *ProgID)
   return hr;
 }
 
-HRESULT tCOMUtil::GetCLSID(ITypeInfo *coclassinfo, CLSID *pClsid)
+CLSID tCOMUtil::GetCLSID(ITypeInfo *coclassinfo)
 {
   TYPEATTR* ptypeattr = NULL;
 
   HRESULT hr = coclassinfo->GetTypeAttr(&ptypeattr);
 
   if(FAILED(hr))
-    return hr;
+    return IID_NULL;
 
-  *pClsid = ptypeattr->guid;
+  CLSID clsid = ptypeattr->guid;
 
   coclassinfo->ReleaseTypeAttr(ptypeattr);
 
-  return hr;
+  return clsid;
 }
 
 bool tCOMUtil::GetDefaultTypeLibVersion(const char* libid,
-                                        unsigned short *version_major,
-                                        unsigned short *version_minor)
+                                        int *version_major,
+                                        int *version_minor)
 {
   LONG res = 0;
   HKEY typelib_key, this_typelib_key;
@@ -600,6 +626,22 @@ bool tCOMUtil::GetDefaultTypeLibVersion(const char* libid,
   return true;
 }
 
+bool tCOMUtil::GetRegKeyValue(const char* key, char** pValue) {
+	LONG ec = 0;
+	HKEY hKey;
+	long cbValue;
+
+	ec = RegQueryValue(HKEY_CLASSES_ROOT,key,NULL,&cbValue);
+
+	if(ERROR_SUCCESS == ec) {
+		*pValue = new char[cbValue+1];
+		ec = RegQueryValue(HKEY_CLASSES_ROOT,key,*pValue,&cbValue);
+  		if(ERROR_SUCCESS == ec)
+			return true;
+	}
+
+	return false;
+}
 
 bool tCOMUtil::SetRegKeyValue(const char *key,
                               const char *subkey,
@@ -653,6 +695,34 @@ bool tCOMUtil::SetRegKeyValue(const char *key,
   return ok;
 }
 
+bool tCOMUtil::DelRegKey(const char *key,
+                         const char *subkey)
+{
+
+  bool ok = false;
+  LONG ec = 0;
+
+  const int bufsize = 10000;
+  char Key[bufsize];
+
+  strncpy(Key, key, bufsize - 1);
+
+  if (NULL != subkey)
+  {
+    strcat(Key, "\\");
+    strcat(Key, subkey);
+  }
+
+  ec = SHDeleteKey(HKEY_CLASSES_ROOT, Key);
+
+  if (ERROR_SUCCESS == ec)
+  {
+	  return true;
+  }
+
+  return false;
+}
+
 void tCOMUtil::DumpTypeInfo(ITypeInfo *typeinfo)
 {
   HRESULT hr = S_OK;
@@ -661,7 +731,7 @@ void tCOMUtil::DumpTypeInfo(ITypeInfo *typeinfo)
   CHECKPARAM(typeinfo);
 
   hr = typeinfo->GetTypeAttr(&pta);
-  CHK_COM_ERR2(hr);
+  CHK_COM_CODE(hr);
 
   // prints IID
   LPOLESTR lpsz = NULL;
@@ -703,7 +773,108 @@ void tCOMUtil::DumpTypeInfo(ITypeInfo *typeinfo)
 }
 
 
-const char* tCOMUtil::getPrintableTypeKind(TYPEKIND tkind)
+const char* tCOMUtil::getPrintableInvokeKind(const INVOKEKIND invkind)
+{
+  switch(invkind)
+  {
+  case INVOKE_PROPERTYGET:
+    return "propget";
+
+  case INVOKE_PROPERTYPUT:
+    return "propput";
+
+  case INVOKE_PROPERTYPUTREF:
+    return "propputref";
+
+  case INVOKE_FUNC:
+    return "func";
+  }
+
+  return NULL;
+}
+
+const char* tCOMUtil::getPrintableTypeDesc(const TYPEDESC& tdesc)
+{
+  static char buffer[200];
+  buffer[0] = '\0';
+
+  switch(tdesc.vt & ~(VT_ARRAY | VT_BYREF))
+  {
+  case VT_VOID:
+    strcat(buffer, "void");
+    break;
+
+  case VT_I2:
+    strcat(buffer, "short");
+    break;
+
+  case VT_I4:
+    strcat(buffer, "long");
+    break;
+
+  case VT_R4:
+    strcat(buffer, "float");
+    break;
+
+  case VT_R8:
+    strcat(buffer, "double");
+    break;
+
+  case VT_CY:
+    strcat(buffer, "CY");
+    break;
+
+  case VT_DATE:
+    strcat(buffer, "DATE");
+    break;
+
+  case VT_BSTR:
+    strcat(buffer, "BSTR");
+    break;
+
+  case VT_DISPATCH:
+    strcat(buffer, "IDispatch*");
+    break;
+
+  case VT_BOOL:
+    strcat(buffer, "VARIANT_BOOL");
+    break;
+
+  case VT_VARIANT:
+    strcat(buffer, "VARIANT");
+    break;
+
+  case VT_UNKNOWN:
+    strcat(buffer, "IUnknown*");
+    break;
+
+  case VT_DECIMAL:
+    strcat(buffer, "Decimal");
+    break;
+
+  case VT_UI1:
+    strcat(buffer, "unsigned char");
+    break;
+
+  case VT_INT:
+    strcat(buffer, "int");
+    break;
+
+  case VT_HRESULT:
+    strcat(buffer, "void");
+	break;
+  }
+
+  if(tdesc.vt & VT_BYREF)
+    strcat(buffer, "*");
+
+  if(tdesc.vt & VT_ARRAY)
+    strcat(buffer, "[]");
+
+  return buffer;
+}
+
+const char* tCOMUtil::getPrintableTypeKind(const TYPEKIND tkind)
 {
   switch(tkind)
   {
@@ -744,3 +915,91 @@ const char* tCOMUtil::getPrintableTypeKind(TYPEKIND tkind)
     break;
   }
 }
+
+
+HRESULT tCOMUtil::GUID2String(GUID& Guid, char** ppGuid)
+{
+  wchar_t* wcGuid = NULL;
+
+  HRESULT hr = StringFromCLSID(Guid, &wcGuid);
+
+  if (FAILED(hr))
+    return hr;
+
+  *ppGuid = new char[wcslen(wcGuid) + 1];
+  wcstombs(*ppGuid, wcGuid, wcslen(wcGuid)+1);
+
+  CoTaskMemFree(wcGuid);
+
+  return S_OK;
+}
+
+CLSID tCOMUtil::FindCLSID(ITypeInfo* interface_typeinfo)
+{
+  ITypeLib* ptypelib    = NULL;
+  ITypeInfo* ptypeinfo  = NULL;
+  long count            = 0;
+  IID iid               = IID_NULL;
+  TYPEATTR* ptypeattr   = NULL;
+  TYPEKIND tkind;
+  bool found            = false;
+  CLSID clsid           = IID_NULL;
+
+  // gets IID
+  interface_typeinfo->GetTypeAttr(&ptypeattr);
+
+  iid = ptypeattr->guid;
+  interface_typeinfo->ReleaseTypeAttr(ptypeattr);
+  ptypeattr = NULL;
+
+  // Gets type library
+  interface_typeinfo->GetContainingTypeLib(&ptypelib, NULL);
+
+  // iterates looking for IID inside some coclass
+  count = ptypelib->GetTypeInfoCount();
+
+  while(count-- && !found)
+  {
+    ptypelib->GetTypeInfoType(count, &tkind);
+
+    if(tkind != TKIND_COCLASS)
+      continue;
+
+    // look inside
+    ptypelib->GetTypeInfo(count, &ptypeinfo);
+
+    // gets counts and clsid
+    ptypeinfo->GetTypeAttr(&ptypeattr);
+    long ifaces_count   = ptypeattr->cImplTypes;
+    clsid = ptypeattr->guid;
+
+    ptypeinfo->ReleaseTypeAttr(ptypeattr);
+    ptypeattr = NULL;
+
+    HREFTYPE RefType;
+    ITypeInfo* piface_typeinfo = NULL;
+
+    while(ifaces_count-- && !found)
+    {
+      ptypeinfo->GetRefTypeOfImplType(ifaces_count, &RefType);
+      ptypeinfo->GetRefTypeInfo(RefType, &piface_typeinfo);
+      piface_typeinfo->GetTypeAttr(&ptypeattr);
+
+      if(IsEqualIID(ptypeattr->guid, iid))
+      {
+        found = true;
+      }
+
+      piface_typeinfo->ReleaseTypeAttr(ptypeattr);
+      ptypeattr = NULL;
+
+      COM_RELEASE(piface_typeinfo);
+    }
+
+    COM_RELEASE(ptypeinfo);
+  }
+
+  COM_RELEASE(ptypelib);
+
+  return clsid;
+}
\ No newline at end of file
