@@ -16,8 +16,6 @@ extern "C"
 #include "LuaCompat.h"
 }
 
-#define CHK_COM_ERR2(x) CHK_COM_ERR(x, tUtil::GetErrorMessage(hr))
-
 #define CONNPOINT_NAME "__LUACOM_CONNPOINT"
 
 
@@ -77,7 +75,7 @@ tLuaCOMEnumConnPoints::~tLuaCOMEnumConnPoints(void)
     // Release all the connection point interface pointers.
     for (i=0; i<cConnPts; i++)
       if (NULL != paConnPts[i])
-        paConnPts[i]->Release();
+        COM_RELEASE(paConnPts[i]);
 
     // Delete the array of interface pointers.
     delete [] paConnPts;
@@ -443,12 +441,10 @@ STDMETHODIMP tLuaCOMEnumConnPoints::Clone(
   Returns:  void
 M---M---M---M---M---M---M---M---M---M---M---M---M---M---M---M---M---M---M-M*/
 tLuaCOMConnPoint::tLuaCOMConnPoint(lua_State *p_L,
-                                   LuaBeans* p_lbeans,
                                    IUnknown* pHostObj)
 {
   // stores lua state
   L = p_L;
-  lbeans = p_lbeans;
 
   // Zero the COM object's reference count.
   cRefs = 0;
@@ -471,6 +467,9 @@ tLuaCOMConnPoint::tLuaCOMConnPoint(lua_State *p_L,
 
   lua_pop(L, 1);
 
+  for(int i = 0; i < ALLOC_CONNECTIONS; i++)
+    sinks[i] = NULL;
+
 
   return;
 }
@@ -479,16 +478,20 @@ tLuaCOMConnPoint::tLuaCOMConnPoint(lua_State *p_L,
 tLuaCOMConnPoint::~tLuaCOMConnPoint(void)
 {
   UINT i;
-  IUnknown* pUnk;
 
   if (NULL != paConnections)
   {
     // Release all the connection sink interface pointers.
     for (i=0; i<uiMaxIndex; i++)
     {
-      pUnk = paConnections[i].pUnk;
-      if (NULL != pUnk)
-        pUnk->Release();
+      if (NULL != paConnections[i].pUnk)
+        paConnections[i].pUnk->Release();
+
+      if (NULL != sinks[i])
+      {
+        sinks[i]->Unlock();
+        sinks[i] = NULL;
+      }
     }
   }
 
@@ -622,46 +625,6 @@ HRESULT tLuaCOMConnPoint::GetSlot(
   if (!bOpen)
   {
     hr = E_OUTOFMEMORY;
-#if 0
-    // We didn't find an existing open slot in the array--it's full.
-    // Expand the array by ALLOC_CONNECTIONS entries and assign the
-    // appropriate output index.
-    paConns = new CONNECTDATA[uiMaxIndex + ALLOC_CONNECTIONS];
-    CHKMALLOC(paConns);
-
-    tLuaCOM **new_sinks = new tLuaCOM*[uiMaxIndex + ALLOC_CONNECTIONS];
-    CHKMALLOC(sinks);
-
-    // Copy the content of the old full array to the new larger array.
-    for (i=0; i<uiMaxIndex; i++)
-    {
-      paConns[i].pUnk = paConnections[i].pUnk;
-      paConns[i].dwCookie = paConnections[i].dwCookie;
-      new_sinks[i] = sinks[i];
-    }
-
-      // Zero (ie mark as empty) the expanded portion of the new array.
-      for (i=uiMaxIndex; i<uiMaxIndex+ALLOC_CONNECTIONS; i++)
-      {
-        paConns[i].pUnk = NULL;
-        paConns[i].dwCookie = 0;
-        new_sinks[i] = NULL;
-      }
-
-      // New larger array is ready--delete the old array.
-      delete [] paConnections;
-      delete sinks;
-
-      // Rig the connection point to use the new larger array.
-      paConnections = paConns;
-      sinks = new_sinks;
-
-      // Assign the output free slot as first entry in new expanded area.
-      *puiFreeSlot = uiMaxIndex;
-
-      // Calculate the new max index.
-      uiMaxIndex += ALLOC_CONNECTIONS;
-#endif
   }
 
   return hr;
@@ -824,7 +787,7 @@ STDMETHODIMP tLuaCOMConnPoint::Advise(
         // stay with ours, as it's where vb will look for the
         // DISPID's anyway
         sinks[uiFreeSlot] = 
-          tLuaCOM::CreateLuaCOM(pISink, NULL, lbeans, source_typeinfo, false);
+          tLuaCOM::CreateLuaCOM(L, pISink, IID_NULL, source_typeinfo);
       }
       catch(class tLuaCOMException& e)
       {
@@ -883,6 +846,8 @@ STDMETHODIMP tLuaCOMConnPoint::Unadvise(
       {
         // Release the sink interface.
         COM_RELEASE(paConnections[uiSlot].pUnk);
+        sinks[uiSlot]->Unlock();
+        sinks[uiSlot] = NULL;
 
         // Mark the array entry as empty.
         paConnections[uiSlot].dwCookie = 0;
@@ -1414,7 +1379,6 @@ STDMETHODIMP tLuaCOMEnumConnections::Clone(
   Returns:  void
 M---M---M---M---M---M---M---M---M---M---M---M---M---M---M---M---M---M---M-M*/
 tLuaCOMConnPointContainer::tLuaCOMConnPointContainer(lua_State* p_L,
-                                                     LuaBeans* p_lbeans,
                                                      IUnknown* p_pUnkOuter)
 {
   HRESULT hr = S_OK;
@@ -1430,29 +1394,29 @@ tLuaCOMConnPointContainer::tLuaCOMConnPointContainer(lua_State* p_L,
   IProvideClassInfo2 *ci2  = NULL;
 
   hr = pUnkOuter->QueryInterface(IID_IProvideClassInfo2, (void **) &ci2); 
-  CHK_COM_ERR2(hr);
+  CHK_COM_CODE(hr);
 
   IID iid;
 
   hr = ci2->GetGUID(GUIDKIND_DEFAULT_SOURCE_DISP_IID, &iid);
-  CHK_COM_ERR2(hr);
+  CHK_COM_CODE(hr);
 
   ITypeInfo *coclassinfo = NULL;
 
   hr = ci2->GetClassInfo(&coclassinfo);
-  CHK_COM_ERR2(hr);  
+  CHK_COM_CODE(hr);  
 
   events_typeinfo = tCOMUtil::GetDefaultInterfaceTypeInfo(coclassinfo, true);
   CHK_LCOM_ERR(events_typeinfo, "No default source typeinfo.");
 
-  ci2->Release();
-  coclassinfo->Release();
+  COM_RELEASE(ci2);
+  COM_RELEASE(coclassinfo);
 
   max_connection_points = 1;
   connection_points = new tLuaCOMConnPoint*[max_connection_points];
   CHKMALLOC(connection_points);
 
-  connection_points[0] = new tLuaCOMConnPoint(L, p_lbeans, p_pUnkOuter);
+  connection_points[0] = new tLuaCOMConnPoint(L, p_pUnkOuter);
   CHKMALLOC(connection_points[0]);
 
   connection_points[0]->AddRef();
@@ -1460,7 +1424,7 @@ tLuaCOMConnPointContainer::tLuaCOMConnPointContainer(lua_State* p_L,
   num_connection_points = 1;
 
   hr = connection_points[0]->Init(iid, events_typeinfo);
-  CHK_COM_ERR2(hr);  
+  CHK_COM_CODE(hr);  
 
   default_connection = connection_points[0];
 
@@ -1473,10 +1437,10 @@ tLuaCOMConnPointContainer::~tLuaCOMConnPointContainer(void)
 {
   while(num_connection_points--)
   {
-    connection_points[num_connection_points]->Release();
+    COM_RELEASE(connection_points[num_connection_points]);
   }
 
-  delete connection_points;
+  delete[] connection_points;
   return;
 }
 
@@ -1686,7 +1650,9 @@ int tLuaCOMConnPoint::call_sinks(lua_State *L)
   for(i = 0; i < (int)cp->cConnections; i++)
   {
     // pushes function
-    cp->lbeans->push(cp->sinks[i]);
+    LuaBeans::push(L, cp->sinks[i]);
+    cp->sinks[i]->Lock();
+
     lua_pushvalue(L, event);
     lua_gettable(L, -2);
     
@@ -1694,7 +1660,8 @@ int tLuaCOMConnPoint::call_sinks(lua_State *L)
     lua_remove(L, -2);
 
     // self param (mandatory but unused)
-    cp->lbeans->push(cp->sinks[i]);
+    LuaBeans::push(L, cp->sinks[i]);
+    cp->sinks[i]->Lock();
 
     // duplicates parameters (if any)
     for(i = user_first_param; i <= user_last_param; i++)
