@@ -21,13 +21,20 @@ class tLuaCOM;
 class tLuaCOMTypeHandler  
 {
 public:
-  void setOutValues(FUNCDESC* pFuncDesc, DISPPARAMS* pDispParams, stkIndex outvalue);
+  void setOutValues(
+    lua_State* L, 
+    FUNCDESC* pFuncDesc,
+    DISPPARAMS* pDispParams,
+    stkIndex outvalue);
+
   void pushLuaArgs(
+    lua_State* L, 
     DISPPARAMS* pDispParams,
     const ELEMDESC* pElemDesc
     );
 
   void fillDispParams(
+    lua_State* L, 
     DISPPARAMS& rDispParams,
     FUNCDESC *pfuncdesc,
     tLuaObjList& params,
@@ -36,16 +43,26 @@ public:
 
   void releaseVariants(DISPPARAMS *pDispParams);
 
-  int pushOutValues(const DISPPARAMS& dispparams);
+  void releaseVariant(VARIANTARG *pvarg, bool release_memory=true);
 
-  bool setRetval(const FUNCDESC *funcdesc, stkIndex luaval, VARIANTARG *pvarg);
+  int pushOutValues(lua_State* L, const DISPPARAMS& dispparams);
 
-  void lua2com(stkIndex luaval, VARIANTARG& varg);
+  bool setRetval(
+    lua_State* L,
+    const FUNCDESC *funcdesc,
+    stkIndex luaval,
+    VARIANTARG *pvarg);
 
-  void com2lua(VARIANTARG varg);
-  tLuaCOMTypeHandler(ITypeInfo *ptypeinfo, LuaBeans *lbeans);
+  void lua2com(lua_State* L, stkIndex luaval, VARIANTARG& varg, VARTYPE type=VT_VARIANT);
+
+  void com2lua(lua_State* L, VARIANTARG varg);
+  tLuaCOMTypeHandler(ITypeInfo *ptypeinfo);
   virtual ~tLuaCOMTypeHandler();
 
+	static TYPEDESC processTYPEDESC(
+    ITypeInfo* typeinfo,
+    TYPEDESC tdesc);
+
 protected:
   long * dimensionsFromBounds(SAFEARRAYBOUND* bounds, long num_bounds);
   void put_in_array(SAFEARRAY* safearray,
@@ -62,29 +79,42 @@ protected:
 
 	void Coerce(VARIANTARG& dest, VARIANTARG src, VARTYPE vt);
 	long VariantSize(VARTYPE vt);
-	TYPEDESC processSAFEARRAY(TYPEDESC& tdesc);
+	static TYPEDESC processSAFEARRAY(ITypeInfo* typeinfo, TYPEDESC& tdesc);
 	void toByRefParam(VARIANT& var_source, VARIANTARG * pvarg_dest);
-	void initByRefParam(VARIANTARG* pvarg, VARTYPE vt);
-	void pushIUnknown(IUnknown *punk);
-	bool isIUnknown(stkIndex value);
-	TYPEDESC processTYPEDESC(TYPEDESC tdesc);
-	TYPEDESC processAliases(const TYPEDESC& tdesc);
-	void releaseVariant(VARIANTARG *pvarg, bool release_memory=true);
-	TYPEDESC processUSERDEFINED(const TYPEDESC& tdesc);
-	TYPEDESC processPTR(const TYPEDESC& tdesc);
-  lua_State *L;
-  tLuaCOM * from_lua(int index);
+	void initByRefParam(VARIANTARG* pvarg, VARTYPE vt, bool alloc_memory = true);
+	void pushIUnknown(lua_State* L, IUnknown *punk);
+	bool isIUnknown(lua_State* L, stkIndex value);
+	static TYPEDESC processAliases(
+    ITypeInfo* typeinfo,
+    const TYPEDESC& tdesc);
+	static TYPEDESC processUSERDEFINED(
+    ITypeInfo* typeinfo,
+    const TYPEDESC& tdesc);
+	static TYPEDESC processPTR(
+    ITypeInfo* typeinfo,
+    const TYPEDESC& tdesc);
+
+  tLuaCOM * from_lua(lua_State* L, int index);
+  tLuaCOM * convert_table(lua_State *L, stkIndex luaval);
   stkIndex get_from_array(
+    lua_State* L, 
     SAFEARRAY* safearray,
     long *indices,
     const VARTYPE& vt);
 
-  void safearray_com2lua(VARIANTARG& varg);
-  void safearray_lua2com(stkIndex luaval, VARIANTARG& varg, VARTYPE vt);
+  void safearray_com2lua(lua_State* L, VARIANTARG& varg);
+
+  void safearray_lua2com(
+    lua_State* L,
+    stkIndex luaval,
+    VARIANTARG& varg, VARTYPE vt,
+    bool from_stack = false
+    );
+
   void string2safearray(const char* str, long len, VARIANTARG& varg);
+  void safearray2string(lua_State* L, VARIANTARG & varg);
 
   ITypeInfo * m_typeinfo;
-  LuaBeans *lbeans;
 };
 
 #define LUACOM_IUNKNOWN_TAGNAME "_LuaCOM_IUnknown_tag"
