@@ -18,7 +18,8 @@ tStringBuffer::tStringBuffer()
 
 tStringBuffer::~tStringBuffer()
 {
-
+  if(buffer!=NULL)
+	  delete[] buffer;
 }
 
 void tStringBuffer::copyToBuffer(char * source)
@@ -28,7 +29,7 @@ void tStringBuffer::copyToBuffer(char * source)
   if(new_size > size)
   {
     if(buffer != NULL)
-      delete buffer;
+      delete[] buffer;
 
     size = new_size;
     buffer = new char[size];
